/**
 * contain score method
 */
public class Score {
	
	public abstract class BasicInfo {
		
		// Number of documents with this term
		protected double num_docTerm;
		
		// Number of documents
		protected double num_doc;
		
		/**
		 * lengthDoc : how many words in the documents
		 * numDocHasTerm : number of documents have this term
		 * numTermInDoc : number of terms in documents
		 * 
		 * Return : score
		 */
		public abstract double getScore(int lengthDoc, int numDocHasTerm, int numTermInDoc);
	}
	
	
	/**
	 * Using BM25 score weighting method
	 * 
	 * Formula : IDF * (c * (1 + k1)) / (c + k1 * ((1-b) + b * (L / avgLength)))
	 * IDF : log((N - f + 0.5) / (0.5 + f))
	 * b and k1 are constant
	 * L : length of documents 
	 * avgLength : average length of documents
	 * c : frequency of term ==>> (number of terms in the document) / (document length)
	 * N : total of documents (num_doc)
	 * f : number of documents with term (num_docTerm)
	 * 
	 * IDF can be refactored into log(N/f) if a term of query occurs in over half the document, in that case, IDF is negative
	 * 
	 * Reference : http://en.wikipedia.org/wiki/BM25
	 */
	public class BM25 extends BasicInfo {

		// Constant value
		private final double k1 = 2.0d;
		private final double b = 0.75d;
		private double avgLength;
		
		// Constructor
		public BM25 (double avgLength, int num_doc) {
			this.avgLength = avgLength;
			this.num_doc = num_doc * 1.0d;
		}
		
		public double getScore(int lengthDoc, int numDocHasTerm, int num_docTerm) {
			
			// if denominator is zero, reset it to 1.0
			if (num_docTerm != 0) {
				this.num_docTerm = num_docTerm * 1.0d;
			}
			else {
				this.num_docTerm = 1.0d;
			}
			
			double c = (numDocHasTerm * 1.0d) / (lengthDoc * 1.0d);
			
			// Refactor IDF with log(N/f)
			double idf = Math.log((this.num_doc) / (this.num_docTerm));
			double nominator = idf * (c * (1 + this.k1));
			double denominator = c + this.k1 * (1 - this.b + this.b * (lengthDoc * 1.0d / this.avgLength));

			return (nominator / denominator);
		}
		
	}
	
}
